import { SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import { rootSelector } from '@/redux/root/root.selectors';
import { BioEntityContent } from '@/types/models';
import { createSelector } from '@reduxjs/toolkit';
import { currentModelIdSelector } from '../models/models.selectors';

export const bioEntitySelector = createSelector(rootSelector, state => state.bioEntity);

export const loadingBioEntityStatusSelector = createSelector(
  bioEntitySelector,
  state => state.loading,
);

export const allBioEntitesSelectorOfCurrentMap = createSelector(
  bioEntitySelector,
  currentModelIdSelector,
  (state, currentModelId): BioEntityContent[] =>
    (state?.data || []).filter(({ bioEntity }) => bioEntity.model === currentModelId),
);

export const numberOfBioEntitiesSelector = createSelector(bioEntitySelector, state =>
  state.data ? state.data.length : SIZE_OF_EMPTY_ARRAY,
);

export const numberOfBioEntitiesPerModelSelector = createSelector(rootSelector, state => {
  const {
    models,
    bioEntity: { data: bioEntities },
  } = state;

  const numberOfBioEntitiesPerModel = (models.data || []).map(model => {
    const bioEntitiesInGivenModel = (bioEntities || []).filter(
      entity => model.idObject === entity.bioEntity.model,
    );

    return { modelName: model.name, numberOfEntities: bioEntitiesInGivenModel.length };
  });

  return numberOfBioEntitiesPerModel.filter(
    model => model.numberOfEntities !== SIZE_OF_EMPTY_ARRAY,
  );
});
