import { bioEntityResponseFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { apiPath } from '@/redux/apiPath';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import contentsReducer from './bioEntity.slice';
import { getBioEntity } from './bioEntity.thunks';
import { BioEntityContentsState } from './bioEntity.types';

const mockedAxiosClient = mockNetworkNewAPIResponse();
const SEARCH_QUERY = 'park7';
const SEARCH_QUERY_IDS = ['1000', '2000'];

describe('bioEntityContents thunks', () => {
  let store = {} as ToolkitStoreWithSingleSlice<BioEntityContentsState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('bioEntityContents', contentsReducer);
  });
  describe('getBioEntityContents', () => {
    describe('on default query search', () => {
      it('should return data when data response from API is valid', async () => {
        mockedAxiosClient
          .onGet(apiPath.getBioEntityContentsStringWithQuery(SEARCH_QUERY))
          .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

        const { payload } = await store.dispatch(getBioEntity(SEARCH_QUERY));
        expect(payload).toEqual(bioEntityResponseFixture.content);
      });
      it('should return undefined when data response from API is not valid ', async () => {
        mockedAxiosClient
          .onGet(apiPath.getBioEntityContentsStringWithQuery(SEARCH_QUERY))
          .reply(HttpStatusCode.Ok, { randomProperty: 'randomValue' });

        const { payload } = await store.dispatch(getBioEntity(SEARCH_QUERY));
        expect(payload).toEqual(undefined);
      });
    });

    describe('on multi query search', () => {
      it('should return data when data response from API is valid', async () => {
        const FIRST = 0;
        const SECOND = 1;
        const params = {
          perfectMatch: true,
        };

        mockedAxiosClient
          .onGet(apiPath.getBioEntityContentsStringWithQuery(SEARCH_QUERY_IDS[FIRST], params))
          .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

        mockedAxiosClient
          .onGet(apiPath.getBioEntityContentsStringWithQuery(SEARCH_QUERY_IDS[SECOND], params))
          .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

        const { payload } = await store.dispatch(
          getBioEntity({
            query: SEARCH_QUERY_IDS,
            params,
          }),
        );

        expect(payload).toEqual([
          ...bioEntityResponseFixture.content,
          ...bioEntityResponseFixture.content,
        ]);
      });
      it('should return undefined when data response from API is not valid ', async () => {
        const FIRST = 0;
        const SECOND = 1;
        const params = {
          perfectMatch: true,
        };

        mockedAxiosClient
          .onGet(apiPath.getBioEntityContentsStringWithQuery(SEARCH_QUERY_IDS[FIRST], params))
          .reply(HttpStatusCode.Ok, bioEntityResponseFixture);

        mockedAxiosClient
          .onGet(apiPath.getBioEntityContentsStringWithQuery(SEARCH_QUERY_IDS[SECOND], params))
          .reply(HttpStatusCode.Ok, { randomProperty: 'randomValue' });

        const { payload } = await store.dispatch(
          getBioEntity({
            query: SEARCH_QUERY_IDS,
            params,
          }),
        );

        expect(payload).toEqual(undefined);
      });
    });
  });
});
