import { bioEntityResponseSchema } from '@/models/bioEntityResponseSchema';
import { apiPath } from '@/redux/apiPath';
import { axiosInstanceNewAPI } from '@/services/api/utils/axiosInstance';
import { BioEntityContent, BioEntityResponse } from '@/types/models';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { DEFAULT_BIOENTITY_PARAMS } from './bioEntity.constants';
import { BioEntityContentSearchQuery } from './bioEntity.types';

const getQueryResponse = async ({
  query,
  params,
}: BioEntityContentSearchQuery): Promise<BioEntityResponse> => {
  const queries = typeof query === 'string' ? [query] : query;
  const responses = await Promise.all(
    queries.map(searchQuery =>
      axiosInstanceNewAPI.get<BioEntityResponse>(
        apiPath.getBioEntityContentsStringWithQuery(searchQuery, params),
      ),
    ),
  );

  const responsesData = responses.map(response => response.data);

  return responsesData.reduce((acc, next) => ({
    ...acc,
    size: acc.size + next.size,
    totalPages: acc.totalPages + next.totalPages,
    totalElements: acc.totalElements + next.totalElements,
    numberOfElements: acc.numberOfElements + next.numberOfElements,
    content: [...acc.content, ...next.content],
  }));
};

export const getBioEntity = createAsyncThunk(
  'project/getBioEntityContents',
  async (
    searchQuery: string | BioEntityContentSearchQuery,
  ): Promise<BioEntityContent[] | undefined> => {
    const query = typeof searchQuery === 'string' ? searchQuery : searchQuery.query;
    const params = typeof searchQuery === 'string' ? DEFAULT_BIOENTITY_PARAMS : searchQuery.params;

    const response = await getQueryResponse({ query, params });
    const isDataValid = validateDataUsingZodSchema(response, bioEntityResponseSchema);

    return isDataValid ? response.content : undefined;
  },
);
