import { reactionSchema } from '@/models/reaction';
import { apiPath } from '@/redux/apiPath';
import { axiosInstance } from '@/services/api/utils/axiosInstance';
import { Reaction } from '@/types/models';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { z } from 'zod';

const ZERO = 0;

export const getReactionsByIds = createAsyncThunk<Reaction[] | undefined, number[]>(
  'reactions/getByIds',
  async (ids: number[]): Promise<Reaction[] | undefined> => {
    const response = await axiosInstance.get<Reaction[]>(apiPath.getReactionsWithIds(ids));
    const isDataValid = validateDataUsingZodSchema(response.data, z.array(reactionSchema));

    if (!isDataValid || response?.data?.length === ZERO) {
      return isDataValid ? response.data : undefined;
    }

    return response.data;
  },
);
